package de.unibremen.ggj2014;

import java.util.Arrays;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

public class Animation extends Updateable  {
    private final static String FALLBACK = "data/notfound.png";
    private final static float DEFAULT_DURATION = 0.15f;

    private Texture texture;
    private com.badlogic.gdx.graphics.g2d.Animation animation;
    private TextureRegion currentFrame;
    private float rotation = 0;

    private float stateTime = 0f;

    public Animation(final String name, int sWidth) {
        this(name, sWidth, DEFAULT_DURATION);
    }

    public Animation(final String name, int sWidth, float duration) {
        FileHandle f = Gdx.files.internal(name);
        if (!f.exists()) {
            f = Gdx.files.internal(FALLBACK);
        }
        texture = new Texture(f);
        TextureRegion[][] regions = TextureRegion.split(texture, sWidth, texture.getHeight());
        TextureRegion[] frames = Arrays.copyOf(regions[0], texture.getWidth() / sWidth);
        animation = new com.badlogic.gdx.graphics.g2d.Animation(duration, frames);
        animation.setPlayMode(animation.LOOP);
    }

    @Override
    public void update(float dTime) {
        stateTime += dTime;
        currentFrame = animation.getKeyFrame(stateTime);
    }
    
    public void disableLoop() {
    	this.animation.setPlayMode(com.badlogic.gdx.graphics.g2d.Animation.NORMAL);
//    	System.out.println(this.animation.getPlayMode() + ", expected " + com.badlogic.gdx.graphics.g2d.Animation.NORMAL);
    }
    
    public void draw(Batch batch, float x, float y) {
        if (currentFrame == null) {
            return;
        }
        batch.draw(currentFrame, x-currentFrame.getRegionWidth() / 2f, y-currentFrame.getRegionHeight() / 2f, currentFrame.getRegionWidth() / 2f, currentFrame.getRegionHeight() / 2f, currentFrame.getRegionWidth(), currentFrame.getRegionHeight(), 1, 1, rotation);
    }
    
    public void setVisualRotation(float rotation) {
    	this.rotation = rotation;
    }

    public float getVisualRotation() {
    	return this.rotation;
    }
    
    public void reset() {
        this.stateTime = 0;
    }
}
