package de.unibremen.ggj2014;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.Contact;
import com.badlogic.gdx.physics.box2d.ContactImpulse;
import com.badlogic.gdx.physics.box2d.ContactListener;
import com.badlogic.gdx.physics.box2d.Manifold;

public class CollideListener implements ContactListener {
    
    public enum CollisionSurface { NORTH, SOUTH, EAST, WEST };
	
    public CollideListener() {
        // TODO Auto-generated constructor stub
    }

    @Override
    public void beginContact(Contact contact) {
        final Body b1 = contact.getFixtureA().getBody();
        final Body b2 = contact.getFixtureB().getBody();
        
        final Entity e1 = (Entity) b1.getUserData();
        final Entity e2 = (Entity) b2.getUserData();
        
        if (e1 instanceof Player) {
            handlePlayerCollision((Player) e1, e2, contact);
        } else if (e2 instanceof Player) {
            handlePlayerCollision((Player) e2, e1, contact);
        } else if (e1 instanceof Particle) {
            handleParticleCollision((Particle) e1, e2);
        } else if (e2 instanceof Particle) {
            handleParticleCollision((Particle) e2, e1);
        } 
    }

    private void handleParticleCollision(Particle particle, Entity e) {
        if (e instanceof Tile) {
            final Tile t = (Tile) e;
            t.addBlood(checkDirection(particle, e)); 
            GGJ2014Game.singleton.soundManager.particle.play();
        }
    }
    
        

	private void handlePlayerCollision(Player player, Entity e, Contact contact) {
		if (player.isDead())
			return;
		if (e instanceof SpawnPosition) {
			GGJ2014Game.singleton.getLevel().setCurrentSpawnPosition(
					(SpawnPosition) e);
		}


	
	if (e instanceof Destination) {
	    GGJ2014Game.singleton.setFinishedLevel();
	}
	
	if (e instanceof Catapult) {
	Body body = player.getBody();
	((Hazard)e).addBlood(CollisionSurface.NORTH); // Always use north!!!
	body.applyLinearImpulse(new Vector2(0f,10f), body.getWorldCenter(), true);
	GGJ2014Game.singleton.soundManager.catapult.play();

	} else if (e instanceof Hazard) {
            ((Hazard) e).addBlood(CollisionSurface.NORTH); // Always use north because they are not direction relative
            final Vector2 v = player.getBody().getLinearVelocity();
            player.die(e, v);
            
            
        } else if (e instanceof Wall){
		switch(checkDirection(e, player)){
		case SOUTH:
			player.setTouchingGround(true);
			player.setBlockedDirection('0');
			break;
		case WEST:
			player.setBlockedDirection('l');
			break;
		case EAST:
			player.setBlockedDirection('r');
			break;
		default:
		}
	}
    }
    
    // 0: e1 ueber e2
    // 1: e1 unter e2
    // 2: e1 links von e2
    // 3: e1 rechts von e2
    public CollisionSurface checkDirection(Entity e1, Entity e2) {
        Vector2 pPos = e1.getBody().getWorldCenter();
        Vector2 ePos = e2.getBody().getWorldCenter();
        
        float angle = (float) Math.toDegrees(Math.atan2(ePos.x - pPos.x, ePos.y - pPos.y)); 

        if(angle < 0){
            angle += 360;
        }
        
        if (angle < 45) {
        	return CollisionSurface.SOUTH;
        }
        else if (angle < 90+45) {
        	return CollisionSurface.WEST;
        }
        else if (angle < 180+45) {
        	return CollisionSurface.NORTH;
        }
        else if (angle < 270+45) {
        	return CollisionSurface.EAST;
        }
        else {
        	return CollisionSurface.SOUTH;
        }
    }

    @Override
    public void endContact(Contact contact) {
        
    }

    @Override
    public void preSolve(Contact contact, Manifold oldManifold) {
        // TODO Auto-generated method stub

    }

    @Override
    public void postSolve(Contact contact, ContactImpulse impulse) {
        // TODO Auto-generated method stub

    }

}
