package de.unibremen.ggj2014;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.concurrent.CopyOnWriteArrayList;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.Filter;

public class Entity extends Updateable implements Drawable {

	public static final int TILE_SIZE_IN_PX = 64;
	
	private ArrayList<Animation> animations = new ArrayList<Animation>();
	private float x, y;
	private Physics physics;

	/**
	 * You have to call setAnimation afterwards!
	 */
	public Entity()
	{
	    init((Animation) null, 0, 0, 1, 1, new Physics(0, 0, 1, 1, false, 0));
	}

	public Entity(final String textureName, Physics physics) {
	    this(new Animation(textureName, TILE_SIZE_IN_PX), physics);
	}

	public Entity (final Animation animation, Physics physics) {
		init(animation, 0, 0, 1, 1, physics);
	}

	private void init(final Animation animation, float x, float y, float width, float height, final Physics physics) {
	    
	    this.setAnimation(animation);
	    assert(physics != null);
        this.physics = physics;
        addBehaviour(this.physics);

		GGJ2014Game.singleton.addDrawable(this);
		GGJ2014Game.singleton.addUpdateable(this);
	}
	
	@Override
	public void init() {
	    super.init();
	    
	    this.x = getBody().getPosition().x;
        this.y = getBody().getPosition().y;
        physics.getBody().setUserData(this);
        
        if(getFilter() != null)
			getBody().getFixtureList().get(0).setFilterData(getFilter());
	}
	
	/**
	 * Sets the x coordinage <b>in tile coordinates</b>.
	 * @param x the new x coordinate
	 */
	public void setX(float x) {
		this.x = x;
	}
	
	/**
	 * Sets the y coordinage <b>in tile coordinates</b>.
	 * @param y the new y coordinate
	 */
	public void setY(float y) {
		this.y = y;
	}
	
	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	@Override
	public void draw(Batch batch) {
	    for (Animation a : animations) {
	        a.draw(batch, x * TILE_SIZE_IN_PX, y * TILE_SIZE_IN_PX);
	    }
	}
	
	@Override
	public void update(float deltaTime) {
        super.update(deltaTime);
        for (Animation a : animations) {
            a.update(deltaTime);
        }
	}

    public Collection<Animation> getAnimations() {
        return new CopyOnWriteArrayList<Animation>(animations);
    }

    public void setAnimation(final Animation animation) {
        this.animations.clear();
        if (animation != null) {
            this.animations.add(animation);
        }
    }
    
    public void setAnimations(final Collection<Animation> animations) {
        this.animations.clear();
        this.animations.ensureCapacity(animations.size());
        for (Animation a : animations) {
            this.animations.add(a);
        }
    }
    
    public void addAnimation(final Animation animation) {
        animations.add(animation);
    }
    
    public Body getBody() {
        return physics.getBody();
    }
    
    public Filter getFilter(){
    	return null;
    }
}
