package de.unibremen.ggj2014;

import java.util.ArrayList;
import java.util.Date;
import java.util.Random;
import java.util.Vector;

import com.badlogic.gdx.ApplicationListener;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.physics.box2d.Box2DDebugRenderer;
import com.badlogic.gdx.physics.box2d.World;

public class GGJ2014Game implements ApplicationListener {
	private OrthographicCamera camera;
	private SpriteBatch batch;
	private Texture texture;
	private Sprite sprite;
	private World world;
	private Box2DDebugRenderer debugRenderer;
	private Music music;
	public Random rnd = new Random();
	public final ParticlePool particles = new ParticlePool();
	
	private int resX;
	private int resY;
	private Matrix4 normalProjection;
	
	private Level level;
	
	static final float BOX_TO_SCREEN = Entity.TILE_SIZE_IN_PX;
	static final float SCREEN_TO_BOX = 1f/BOX_TO_SCREEN;
	
	static final boolean RENDER_BOX2D_DEBUG_DATA = false;
	
	public static GGJ2014Game singleton;
	
	private final Vector<Drawable> drawables = new Vector<Drawable>();
	private final Vector<Updateable> updateables = new Vector<Updateable>();
	private final ArrayList<Runnable> physicsUpdates = new ArrayList<Runnable>();
	private final ArrayList<Updateable> newUpdateables = new ArrayList<Updateable>();
	private final long oldTimestamp = new Date().getTime();
	
	private final String[] levelList = new String[] {
		"lvl/01.lvl", "lvl/02.lvl", "lvl/03.lvl", "lvl/04.lvl", "lvl/05.lvl", "lvl/06.lvl", "lvl/07.lvl", "lvl/08.lvl", "lvl/09.lvl"};
	
	private int lvlidx = 0;
	
	public Entity cameraCenter;
	
	private Player player;

	public SoundManager soundManager;
	private boolean finishedLevel = false;
	private boolean showScores = false;
	private float scoreDelay = -1f;
	
	private ScoreScreen scoreScreen;
	ScreenButton title;
	ScreenText anyKey;
	public boolean gameStarted = false;
	private final Date gameStartedAt = new Date();
	
	@Override
	public void create() {
	    
		if (singleton != null)
			throw new RuntimeException("more than one singleton");
		singleton = this;
		
		world = new World(new Vector2(0, -15), true);
		
		float w = Gdx.graphics.getWidth();
		float h = Gdx.graphics.getHeight();
		
		camera = new OrthographicCamera(1, h/w);
		batch = new SpriteBatch();
		
		
		
		texture = new Texture(Gdx.files.internal("data/Title.png"));
		texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		
//		TextureRegion region = new TextureRegion(texture, 0, 0, 256, 256);
		
		title = new ScreenButton("data/Title.png", Gdx.graphics.getWidth() / 2 - 128, Gdx.graphics.getHeight() / 2 - 128);
		anyKey = new ScreenText("Press any key to start...", Gdx.graphics.getWidth() / 2 - 230, Gdx.graphics.getHeight() / 2 - 128);
		
		//Entity.generateTestEntities();
		
		world.setContactListener(new CollideListener());
      
		if(RENDER_BOX2D_DEBUG_DATA)
				debugRenderer = new Box2DDebugRenderer();
		
		music = Gdx.audio.newMusic(Gdx.files.internal("music/tofusAndTrolls.mp3"));
		music.setLooping(true);
		music.play();
		
		soundManager = SoundManager.getInstance();
		
		scoreScreen = new ScoreScreen();
	}

	public void startGame() {
		if (!gameStarted) {
			gameStarted = true;
			try {
				level = Level.buildFromFile(levelList[0]);
				initNewUpdateables();
				player = Player.spawn();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private void initNewUpdateables() {
	    for (Runnable r : physicsUpdates) {
	        r.run();
	    }
	    physicsUpdates.clear();
	    
	    for (Updateable u : newUpdateables) {
	        u.init();
	        updateables.add(u);
	    }
	    newUpdateables.clear();
	}

	@Override
	public void dispose() {
		batch.dispose();
		texture.dispose();
	}

	@Override
	public void render() {
		float deltaTime = Gdx.graphics.getDeltaTime();
		if (!gameStarted && (Gdx.input.isKeyPressed(Keys.ANY_KEY) || Gdx.input.isTouched())) {// && new Date().getTime() - gameStartedAt.getTime() > 5000)
			anyKey.disable();
			startGame();
		}
		
		title.setY(title.getY() + 1);
		if(Gdx.graphics.getWidth() != resX || Gdx.graphics.getHeight() != resY){
			resX = Gdx.graphics.getWidth();
			resY = Gdx.graphics.getHeight();
			normalProjection = new Matrix4().setToOrtho2D(0, 0, resX, resY);
			UIElement.updateScreenSize(resX, resY);
		}
		if(showScores){
			scoreDelay -= deltaTime;
			if(scoreDelay < 0 && (Gdx.input.isKeyPressed(Input.Keys.ANY_KEY) || Gdx.input.isTouched())){
				nextLevel();
			}
		}
	    if(finishedLevel) {
		this.finishedLevel = false;
		//this.nextLevel();
		finishLevel();
		return;
	    } else {
	    initNewUpdateables();
	    
	    if (Gdx.input.isKeyPressed(Input.Keys.ESCAPE)) {
		Gdx.app.exit();
	    }
	    
	    
		// ########## update ##########
        world.step(deltaTime, 6, 2);
		for (Updateable u : this.updateables) {
			u.update(Gdx.graphics.getDeltaTime());
		}

		if (cameraCenter != null) {
			Vector3 current = new Vector3(cameraCenter.getX() * Entity.TILE_SIZE_IN_PX, cameraCenter.getY() * Entity.TILE_SIZE_IN_PX, camera.position.z);
			Vector3 old = camera.position;
			camera.translate(current.sub(old).scl(0.1f));
			camera.update();
		}
		
		// ########## draw ##########
		Gdx.gl.glClearColor(1, 1, 1, 1);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
		
		batch.setProjectionMatrix(camera.combined);
		batch.begin();
		
		for (Drawable d : this.drawables) {
			d.draw(this.batch);
		}
		
		batch.setProjectionMatrix(normalProjection);
		UIElement.drawAll(this.batch);
		batch.end();
		
//		System.out.println("drew " + this.drawables.size() + " drawables");
		
		if(RENDER_BOX2D_DEBUG_DATA)
			debugRenderer.render(world, new Matrix4(camera.combined).scale(BOX_TO_SCREEN, BOX_TO_SCREEN, 1f));
	}
	}

	@Override
	public void resize(int width, int height) {
		this.camera.viewportHeight = height;
		this.camera.viewportWidth = width;
		this.camera.update();
	}

	@Override
	public void pause() {
	}

	@Override
	public void resume() {
	}
	
	public void addDrawable(Drawable d) {
		this.drawables.add(d);
		//System.out.println("Added drawable " + d.getClass());
	}
	
	public void addUpdateable(Updateable u) {
		this.newUpdateables.add(u);
	}
	
	public World getWorld(){
		return world;
	}
	
	public Level getLevel() {
	    return this.level;
	}
	
	public void addPhysicsUpdate(Runnable r) {
	    physicsUpdates.add(r);
	}
	
	public static float rand(float min, float max) {
        return min + (float) Math.random() * (max - min);
	}
	
	public void setFinishedLevel() {
	    this.finishedLevel  = true;
	}
	
	
	private void finishLevel() {
		this.drawables.clear();
	    this.updateables.clear();
	    this.physicsUpdates.clear();
	    this.newUpdateables.clear();
	    this.particles.clear();
		scoreScreen.setClock(player.getTimeInLevel());
		scoreScreen.setDeaths(player.getLifesInLevel());
		scoreScreen.setLevelName(String.format("%d", lvlidx+1));
		scoreScreen.show();
		scoreDelay = 1f;
		showScores = true;
	}
	
	private void nextLevel() {
		scoreScreen.hide();
		showScores = false;
	    world = new World(new Vector2(0, -15), true);
	    world.setContactListener(new CollideListener());
	    
	    lvlidx++;
	    
	    if(lvlidx >= (this.levelList.length)) {
		lvlidx = 0;
		// TODO: Game over Screen / scores?
	    }
	    
	    try {
		this.level = Level.buildFromFile(this.levelList[lvlidx]);
		initNewUpdateables();
		player = Player.spawn();
	    } catch (Exception e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	}
	
	public void moveLastDrawableToFront() {
		this.drawables.insertElementAt(this.drawables.lastElement(), 0);
		this.drawables.removeElementAt(this.drawables.size() - 1);
	}
}
