package de.unibremen.ggj2014;

import com.badlogic.gdx.physics.box2d.Filter;

public class Hazard extends Tile {
    
    public enum Alignment { NORTH, SOUTH, EAST, WEST };
    
    private Alignment myAlignment = null;

	public Hazard(Animation animation, float x, float y, float width, float height) {
		super(animation, x, y, width, height);
	}
	
	public Hazard(Animation animation, float x, float y, float r) {
		super(animation, x, y, r);
	}
	
	public void setAlignment(Alignment a) {
	    this.myAlignment = a;
	}
	
	/**
	 * Returns Hazard.Alignment enum (NORTH, SOUTH, EAST or WEST or null, null means no Alignment!)
	 * @return
	 */
	public Alignment getAlignment() {
	    return this.myAlignment;
	}
	
	@Override
	public void init() {
		super.init();
		getBody().getFixtureList().get(0).setSensor(true);
	}
	
	private static Filter filter;
	@Override
	public Filter getFilter() {
		if(filter == null){
	    	filter = new Filter();
	    	filter.categoryBits = Physics.HAZARD; // I am a
	    	filter.maskBits = Physics.PLAYER | Physics.PARTICLE; // I can collide with
	    }
		return filter;
	}
	
	public void setAlignmentRotation() {
	    Alignment alignment = this.getAlignment();
	    if(alignment == null) return;
	    float rotation = 0f;
	    switch (alignment) {
	    	case NORTH: 
	    		rotation = 180f;
	    		break;
	    	case SOUTH:
	    		rotation = 0f;
	    		break;
	    	case WEST:
	    		rotation = 270f;
	    		break;
	    	case EAST:
	    		rotation = 90f;
	    		break;
	    }
    	for (Animation a : this.getAnimations()) {
			a.setVisualRotation(rotation);
		}
	}

}
