package de.unibremen.ggj2014;

import java.util.LinkedList;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.g2d.Batch;
public class Level extends Updateable implements Drawable {

	private final int height;
	private final int width;

	// 
	private final LinkedList<Entity> entities;

	// Current spawn position
	private SpawnPosition currentSpawnPosition;

	private final int bodyCount;


	private Level(int height, int width, LinkedList<Entity> entities, SpawnPosition p) {

		this.height = height;
		this.width = width;
		this.entities = entities;
		this.currentSpawnPosition = p;

		this.bodyCount = 0;

	}

	//TODO
	public static Level buildFromFile(String filename) throws Exception {

		LinkedList<Entity> entities = new LinkedList<Entity>();
		SpawnPosition currentSpawnPosition = null;

		FileHandle fh = Gdx.files.internal(filename);

		String rawlvl = fh.readString();

//		String[] rawlvllines = rawlvl.split(System.getProperty("line.separator"));
		String[] rawlvllines = rawlvl.split("\\r?\\n");

		int height = rawlvllines.length;
		int width = 0;

		for(int i = 0; i < height; i++) {
		    	String line = rawlvllines[i];

			if(width < line.length()) {
				width = line.length();
			}



			for(int j = 0; j < line.length(); j++) {
			    
			    char c = line.charAt(j);
			    
				if(String.valueOf(c).equals("#")) {

					Wall w = new Wall(j, height - i);
					entities.add(w);
				} else if(String.valueOf(c).equals("P")) {
					SpawnPosition p = new SpawnPosition(j, height-i);
					entities.add(p);
					if(currentSpawnPosition != null) {
						throw new Exception("Only one start position P allowed per level!");
					} else {
						currentSpawnPosition = p;
					}
				} else if(String.valueOf(c).equals("S")) {
					SpawnPosition p = new SpawnPosition(j, height-i);
					entities.add(p);
				} else if (String.valueOf(c).equals("o")) {
				    Saw s = new Saw(j, height-i); 
				    findAlignment(s, rawlvllines, i, j);
					    
				} else if (String.valueOf(c).equals("s")) {
				    SingleSpike s = new SingleSpike(j, height-i); 
				    findAlignment(s, rawlvllines, i, j);
				    
				} else if (String.valueOf(c).equals("m")) {
				    Masher m = new Masher(j, height-i); 
				    findAlignment(m, rawlvllines, i, j);
				    
				} else if (String.valueOf(c).equals("b")) {
				    BearTrap b = new BearTrap(j, height-i); 
				    findAlignment(b, rawlvllines, i, j);
				    
				} else if (String.valueOf(c).equals("c")) {
				    Catapult cat = new Catapult(j, height-i); 
				    findAlignment(cat, rawlvllines, i, j);
				    
				} else if (String.valueOf(c).equals("D")) {
				    Destination d = new Destination(j, height-i);
				    
				} else if (String.valueOf(c).equals("O")) {
					GiantSaw s = new GiantSaw(j, height-i); 
					findAlignment(s, rawlvllines, i, j);
					GGJ2014Game.singleton.moveLastDrawableToFront();
				}
			}
		}

		if(currentSpawnPosition == null) {
			throw new Exception("No start position defined!!!");
		} else {
			return new Level(height, width, entities, currentSpawnPosition);

		}
	}
	
	private static void findAlignment(Hazard h, String[] rawlvllines, int i, int j) {
	    if(rawlvllines.length > i+1 && String.valueOf(rawlvllines[i+1].charAt(j)).equals("#")) {
		h.setAlignment(Hazard.Alignment.SOUTH);
	    } else if (0 <= i-1 && String.valueOf(rawlvllines[i-1].charAt(j)).equals("#")) {
		h.setAlignment(Hazard.Alignment.NORTH);
	    } else if (rawlvllines[i].length() > j+1 && String.valueOf(rawlvllines[i].charAt(j+1)).equals("#")) {
		h.setAlignment(Hazard.Alignment.EAST);
	    } else if (0 <= j-1 && String.valueOf(rawlvllines[i].charAt(j-1)).equals("#")) {
		h.setAlignment(Hazard.Alignment.WEST);
	    }
	}

	@Override
	public void draw(Batch batch) {
	    // TODO Auto-generated method stub
	    
	}
	
	public SpawnPosition getCurrentSpawnPosition() {
	    return this.currentSpawnPosition;
	}
	
	public void setCurrentSpawnPosition(SpawnPosition p) {
	    this.currentSpawnPosition = p;
	}
}
