package de.unibremen.ggj2014;

import java.util.Arrays;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;

import de.unibremen.ggj2014.BloodSplash.Direction;

public class ParticlePool {
    public static final int MAX_PARTICLES = 256;
    
    private final Particle[] particles = new Particle[MAX_PARTICLES];
    private int idx = 0;
    
    private BloodSplash bsLeft;
    private BloodSplash bsRight;
    
    public Particle getParticle(final float x, final float y, final Vector2 v) {
        Particle p = null;
        if (particles[idx] == null) {
            // create new particle
            int goreType = GGJ2014Game.singleton.rnd.nextInt(4);
            if (goreType == 0) {
                p = new Particle("data/GoreParticle.png", x, y, 0.1f, v);
            } else if (goreType == 1) {
                p = new Particle("data/GoreParticle2.png", x, y, 0.05f, v);
            } else if (goreType == 2) {
                p = new Particle("data/GoreParticle3.png", x, y, 0.05f, v);
            } else {
                p = new Particle("data/GoreParticle4.png", x, y, 0.05f, v);
            }
            particles[idx] = p;
        } else {
            // reuse particle
            p = particles[idx];
            final Body b = p.getBody();
            GGJ2014Game.singleton.addPhysicsUpdate(new Runnable() {
                @Override
                public void run() {
                    b.setTransform(x, y, 0);
                    b.setLinearVelocity(v);
                }
            });
        }
        
        // increment
        if (idx + 1 == particles.length) {
            idx = 0;
        } else {
            ++idx;
        }
        
        return p; 
    }
    
    public BloodSplash getBloodSplash(BloodSplash.Direction d, float x, float y) {
        switch (d) {
            case LEFT:
                bsLeft = getBs(bsLeft, d, x, y);
                return bsLeft;
            case RIGHT:
                bsRight = getBs(bsRight, d, x, y);
                return bsRight;
        }
        assert false;
        return null;
    }

    private BloodSplash getBs(BloodSplash bs, Direction d, final float x, final float y) {
        if (bs == null) {
            return new BloodSplash(d, x, y);
        } else {
            final Body b = bs.getBody();
            GGJ2014Game.singleton.addPhysicsUpdate(new Runnable() {
                @Override
                public void run() {
                    if(b == null) {
                	return;
                    }
                    b.setTransform(x, y, 0);
                }
            });
            bs.getAnimations().iterator().next().reset();
            return bs;
        }
    }
    
    public void clear() {
        bsLeft = null;
        bsRight = null;
        idx = 0;
        Arrays.fill(particles, null);
    }
}
