package de.unibremen.ggj2014;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.CircleShape;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.physics.box2d.Shape;

public class Physics extends Behaviour {
	
	public final static int GROUND = 2;
	public final static int PLAYER = 4;
	public final static int HAZARD = 8;
	public final static int PARTICLE = 16;
	public final static int TRIGGER = 32;

	Body body;
	
	private void init(final float x, final float y, final Shape shape, final boolean dynamic, final float bouncyness){
	    GGJ2014Game.singleton.addPhysicsUpdate(new Runnable() {
            @Override
            public void run() {
             // First we create a body definition
                BodyDef bodyDef = new BodyDef();
                if(dynamic){
                    // We set our body to dynamic, for something like ground which doesn't move we would set it to StaticBody
                    bodyDef.type = BodyType.DynamicBody;
                }
                // Set our body's starting position in the world
                bodyDef.position.set(x, y);
                
                bodyDef.fixedRotation = true;

                // Create our body in the world using our body definition
                body = GGJ2014Game.singleton.getWorld().createBody(bodyDef);

                // Create a fixture definition to apply our shape to
                FixtureDef fixtureDef = new FixtureDef();
                fixtureDef.shape = shape;
                fixtureDef.density = 0.5f; 
                fixtureDef.friction = 0.4f;
                fixtureDef.restitution = bouncyness; // Make it bounce a little bit

                // Create our fixture and attach it to the body
                body.createFixture(fixtureDef);
                
                shape.dispose();
            }
        });
		
	}
	
	Physics(float x, float y, float radius, boolean dynamic, float bouncyness){
		CircleShape circle = new CircleShape();
		circle.setRadius(radius);

		init(x, y, circle, dynamic, bouncyness);
	}
	
	Physics(float x, float y, float width, float height, boolean dynamic, float bouncyness){
		PolygonShape rect = new PolygonShape();
		Vector2[] vertices = new Vector2[4];
		vertices[0] = new Vector2(-width/2, -height/2);
		vertices[1] = new Vector2(width/2, -height/2);
		vertices[2] = new Vector2(width/2, height/2);
		vertices[3] = new Vector2(-width/2, height/2);
		rect.set(vertices);
		
		init(x, y, rect, dynamic, bouncyness);
	}
	
	@Override
	public void run(Updateable u, float deltaTime) {
		Entity e = (Entity) u;
		e.setX(body.getPosition().x);
		e.setY(body.getPosition().y);
		//TODO: rotation
	}

	Body getBody(){
		return body;
	}
}
