package de.unibremen.ggj2014;

import java.util.LinkedList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;

public abstract class UIElement {

	private static List<UIElement> items;
	private int x, y, absoluteX, absoluteY, width, height;
	
	public interface UICallback {
		void isClicked();
	}
	
	private UICallback callback;
	
	public void setCallback(UICallback callback) {
		this.callback = callback;
	}

	public UIElement(int x, int y, int width, int height) {
		if(items == null){
			items = new LinkedList<UIElement>();
		}
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		updateScreenResItem(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
		items.add(this);
	}
	
	private void updateScreenResItem(int screenWidth, int screenHeight){
		if(x < 0){
			absoluteX = screenWidth - width + x;
		} else {
			absoluteX = x;
		}
		if(y < 0){
			absoluteY = screenHeight - height + y;
		} else {
			absoluteY = y;
		}
	}
	
	private boolean isClicked() {
		if (Gdx.input.isButtonPressed(Input.Buttons.LEFT)) {
			int xClick = Gdx.input.getX();
			int yClick = Gdx.graphics.getHeight()-Gdx.input.getY();

			if (xClick >= x && xClick <= x + width && yClick >= y
					&& yClick <= y + height)
				return true;
			return false;
		}
		return false;
	}
	
	public int getX() {
		return x;
	}

	public void setX(int x) {
		this.x = x;
		updateScreenResItem(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	public int getY() {
		return y;
	}

	public void setY(int y) {
		this.y = y;
		updateScreenResItem(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
		updateScreenResItem(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
		updateScreenResItem(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	public static void drawAll(SpriteBatch batch){
		if(items == null) return;
		for(UIElement e : items){
			e.drawItem(batch, e.absoluteX, e.absoluteY);
		}
	}
	
	public void drawItem(SpriteBatch batch, int x, int y) {
		if (isClicked() && callback != null) {
			callback.isClicked();
		}
	}

	public static void updateScreenSize(int resX, int resY) {
		if(items == null) return;
		for(UIElement e : items){
			e.updateScreenResItem(resX, resY);
		}
	}
	
	public void disable(){
		items.remove(this);
	}
	
	public void enable(){
		disable();
		items.add(this);
	}
}
